@@ -4,16 +4,18 @@ import warnings
 import h5py
 import nrefocus
 import numpy as np
+import qpretrieve
 from skimage.restoration import unwrap_phase
 
-from . import holo
 from .image_data import Amplitude, Phase, write_image_dataset
 from .meta import MetaDict, DATA_KEYS, META_KEYS
 from ._version import version as __version__
 
 #: valid combinations for keyword argument `which_data`
 VALID_INPUT_DATA = ["field",
-                    "hologram",
+                    "hologram",  # deprecated
+                    "raw-oah",  # off-axis holography
+                    "raw-qlsi",  # quadri-wave lateral shearing interferometry
                     "phase",
                     ("phase", "amplitude"),
                     ("phase", "intensity"),
@@ -21,12 +23,12 @@ VALID_INPUT_DATA = ["field",
 
 
 class QPImage(object):
-    # required to create in-memory hdf5 files with unique fd
+    # required to create in-memory HDF5 files with unique fd
     _instances = 0
 
     def __init__(self, data=None, bg_data=None, which_data="phase",
-                 meta_data=None, holo_kw=None, proc_phase=True,
-                 h5file=None, h5mode="a", h5dtype="float32"):
+                 meta_data=None, qpretrieve_kw=None, holo_kw=None,
+                 proc_phase=True, h5file=None, h5mode="a", h5dtype="float32"):
         """Quantitative phase image manipulation
 
         This class implements various tasks for quantitative phase
@@ -42,17 +44,20 @@ class QPImage(object):
         which_data: str or tuple
             String or comma-separated list of strings indicating
             the order and type of input data. Valid values are
-            "hologram", "field", "phase", "phase,amplitude",
-            or "phase,intensity", where the latter two require an
-            indexable object with the phase data as first element.
+            defined in :const:`VALID_INPUT_DATA`, where phase and
+            amplitude/intensity data require an indexable object
+            with the phase data as first element.
         meta_data: dict or qpimage.MetaDict
             Metadata associated with the input data.
             see :data:`qpimage.meta.META_KEYS`
+        qpretrieve_kw: dict
+            Keyword arguments passed to
+            :ref:`qpretrieve <qpretrieve:index>` for
+            phase retrieval from interferometric data.
         holo_kw: dict
+            This is deprecated, please use `qpretrieve_kw` instead.
             Special keyword arguments for phase retrieval from
-            hologram data (`which_data="hologram"`).
-            See :func:`qpimage.holo.get_field` for valid keyword
-            arguments.
+            hologram data (`which_data="raw-oah"`).
 
             .. versionadded:: 0.1.6
         proc_phase: bool
@@ -65,7 +70,7 @@ class QPImage(object):
                 Previous versions always performed phase unwrapping
                 and did so without offset correction
         h5file: str, pathlib.Path, h5py.Group, h5py.File, or None
-            A path to an hdf5 data file where all data is cached. If
+            A path to an HDF5 data file where all data is cached. If
             set to `None` (default), all data will be handled in
             memory using the "core" driver of the :mod:`h5py`'s
             :class:`h5py:File` class. If the file does not exist,
@@ -98,8 +103,26 @@ class QPImage(object):
             qpi = QPImage(data=...)
             qpi_scliced = qpi[10:20, 40:30]
         """
-        if holo_kw is None:
-            holo_kw = {}
+        if qpretrieve_kw is None:
+            qpretrieve_kw = {}
+        if holo_kw is not None:
+            warnings.warn(
+                "`holo_kw` is deprecated! Please use `qpretrieve_kw` instead",
+                DeprecationWarning)
+            # map deprecated parameters to `qpretrieve_kw`
+            for key in holo_kw:
+                if key == "sideband":
+                    if holo_kw[key] in [-1, 1]:
+                        qpretrieve_kw["sideband_freq"] = None
+                        qpretrieve_kw["invert_phase"] = holo_kw[key] == 1
+                    else:
+                        qpretrieve_kw["sideband_freq"] = holo_kw[key]
+                        qpretrieve_kw["invert_phase"] = False
+                elif key == "zero_pad":
+                    qpretrieve_kw["padding"] = holo_kw["zero_pad"]
+                else:
+                    qpretrieve_kw[key] = holo_kw[key]
+
         if meta_data is None:
             meta_data = {}
         if (data is not None and
@@ -124,7 +147,7 @@ class QPImage(object):
             self._do_h5_cleanup = True
         QPImage._instances += 1
         #: hologram processing keyword arguments
-        self.holo_kw = holo_kw
+        self.qpretrieve_kw = qpretrieve_kw
         # set meta data
         meta = MetaDict(meta_data)
         for key in meta:
@@ -263,9 +286,9 @@ class QPImage(object):
         which_data: str
             String or comma-separated list of strings indicating
             the order and type of input data. Valid values are
-            "field", "phase", "hologram", "phase,amplitude", or
-            "phase,intensity", where the latter two require an
-            indexable object with the phase data as first element.
+            defined in :const:`VALID_INPUT_DATA`, where phase and
+            amplitude/intensity data require an indexable object
+            with the phase data as first element.
         proc_phase: bool
             Process the phase data. This includes phase unwrapping
             using :func:`skimage.restoration.unwrap_phase` and
@@ -294,8 +317,13 @@ class QPImage(object):
         elif which_data == ("phase", "intensity"):
             amp = np.sqrt(data[1])
             pha = data[0]
-        elif which_data == "hologram":
-            amp, pha = self._get_amp_pha(holo.get_field(data, **self.holo_kw),
+        elif which_data in ["hologram", "raw-oah"]:
+            if which_data == "hologram":
+                warnings.warn("The 'hologram' data type is deprecated, "
+                              + "please use 'raw-oah' instead!",
+                              DeprecationWarning)
+            oah = qpretrieve.OffAxisHologram(data, **self.qpretrieve_kw)
+            amp, pha = self._get_amp_pha(oah.run_pipeline(),
                                          which_data="field")
         else:
             raise ValueError(
@@ -357,7 +385,7 @@ class QPImage(object):
 
     @property
     def info(self):
-        """list of tuples with QPImage meta data"""
+        """list of tuples with QPImage metadata"""
         info = []
         # meta data
         meta = self.meta
@@ -533,7 +561,7 @@ class QPImage(object):
     def copy(self, h5file=None):
         """Create a copy of the current instance
 
-        This is done by recursively copying the underlying hdf5 data.
+        This is done by recursively copying the underlying HDF5 data.
 
         Parameters
         ----------
@@ -555,13 +583,13 @@ class QPImage(object):
         kernel: str
             Refocusing method, one of ["helmholtz","fresnel"]
         padding: bool
-            Whether or not to perform padding during refocusing.
+            Whether to perform padding during refocusing.
             You may disable padding if your input image does not
             have any discontinuities at the border (i.e. you can
-            tile your input image and it would look good), otherwise
+            tile your input image, and it would look good), otherwise
             you will experience ringing artifacts.
         h5file: str, h5py.Group, h5py.File, or None
-            A path to an hdf5 data file where the QPImage is cached.
+            A path to an HDF5 data file where the QPImage is cached.
             If set to `None` (default), all data will be handled in
             memory using the "core" driver of the :mod:`h5py`'s
             :class:`h5py:File` class. If the file does not exist,
@@ -651,6 +679,11 @@ class QPImage(object):
             "field", "phase", "phase,amplitude", or "phase,intensity",
             where the latter two require an indexable object for
             `bg_data` with the phase data as first element.
+        proc_phase: bool
+            Process the phase data. This includes phase unwrapping
+            using :func:`skimage.restoration.unwrap_phase` and
+            correcting for 2PI phase offsets (The offset is estimated
+            from a 1px-wide border around the image).
         """
         if isinstance(bg_data, QPImage):
             if which_data is not None:
@@ -671,18 +704,18 @@ class QPImage(object):
 
 
 def copyh5(inh5, outh5):
-    """Recursively copy all hdf5 data from one group to another
+    """Recursively copy all HDF5 data from one group to another
 
     Data from links is copied.
 
     Parameters
     ----------
     inh5: str, h5py.File, or h5py.Group
-        The input hdf5 data. This can be either a file name or
-        an hdf5 object.
+        The input HDF5 data. This can be either a file name or
+        an HDF5 object.
     outh5: str, h5py.File, h5py.Group, or None
-        The output hdf5 data. This can be either a file name or
-        an hdf5 object. If set to `None`, a new hdf5 object is
+        The output HDF5 data. This can be either a file name or
+        an HDF5 object. If set to `None`, a new HDF5 object is
         created in memory.
 
     Notes
@@ -732,7 +765,7 @@ def copyh5(inh5, outh5):
 
 
 def divmod_neg(a, b):
-    """Return divmod with closest result to zero"""
+    """Return divmod with the closest result to zero"""
     q, r = divmod(a, b)
     # make sure r is close to zero
     sr = np.sign(r)
